package javassist.gluonj.pc;

import java.lang.reflect.Method;

import javassist.CtClass;
import javassist.gluonj.WeaveException;
import javassist.gluonj.util.SimplePcNode;

public class PrepareVisitor implements PointcutVisitor {
    protected PatternParser parser;

    public PrepareVisitor(PatternParser p) {
        parser = p;
    }

    public PatternParser getParser() { return parser; }

    public void visit(AndPc n) throws WeaveException {
        n.first().accept(this);
        AndPc r = n.rest();
        if (r != null)
            r.accept(this);
    }

    public void visit(OrPc n) throws WeaveException {
        n.first().accept(this);
        OrPc r = n.rest();
        if (r != null)
            r.accept(this);
    }

    public void visit(NotPc n) throws WeaveException {
        n.getExpr().accept(this);
    }

    public void visit(WithinPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(CallPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(GetPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(SetPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(IfPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(CflowPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(SimplePcNode pc) throws WeaveException {
        if (overriddenInspect(pc))
            parser.getGlue().registerInspector(pc);

        pc.prepare(parser);
    }

    public static boolean overriddenInspect(SimplePcNode pc) {
        try {
            Method m = pc.getClass().getMethod("inspect", new Class[] { CtClass.class });
            if (!m.getDeclaringClass().equals(SimplePcNode.class))
                return true;
        }
        catch (NoSuchMethodException e) {}
        return false;
    }
}
