package javassist.gluonj.weave;

import java.util.List;

import javassist.CtBehavior;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.AndPc;
import javassist.gluonj.pc.CallPc;
import javassist.gluonj.pc.CflowPc;
import javassist.gluonj.pc.GetPc;
import javassist.gluonj.pc.IfPc;
import javassist.gluonj.pc.NotPc;
import javassist.gluonj.pc.OrPc;
import javassist.gluonj.pc.PointcutVisitor;
import javassist.gluonj.pc.SetPc;
import javassist.gluonj.pc.WithinPc;
import javassist.gluonj.util.SimplePcNode;

/**
 * CflowCollector collects all CflowPc nodes in a pointcut
 * expression.  Those nodes are used for inserting code
 * fragments for monitoring control flow.
 *
 * @see CflowPc#instrument(CtBehavior)
 */
public class CflowCollector implements PointcutVisitor {
    protected List cflows;

    public CflowCollector(List list) {
        cflows = list;
    }

    public List getCflows() { return cflows; }

    public void visit(AndPc node) throws WeaveException {
        node.first().accept(this);
        AndPc rest = node.rest();
        if (rest != null)
            rest.accept(this);
    }

    public void visit(OrPc node) throws WeaveException {
        node.first().accept(this);
        OrPc rest = node.rest();
        if (rest != null)
            rest.accept(this);
    }

    public void visit(NotPc n) throws WeaveException {
        n.getExpr().accept(this);
    }

    public void visit(CflowPc node) throws WeaveException {
        cflows.add(node);
    }
    
    public void visit(WithinPc n) throws WeaveException {}
    public void visit(CallPc n) throws WeaveException {}
    public void visit(GetPc n) throws WeaveException {}
    public void visit(SetPc n) throws WeaveException {}
    public void visit(IfPc n) throws WeaveException {}
    public void visit(SimplePcNode pc) throws WeaveException {}
}
