/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;


/**
 * A pair of advice and residue, that is, a hook code that must be inserted
 * at a join point.
 *
 * Gluon#match(..) returns a list of Hook objects (a Hook.List object).
 */
public class Hook {
    Residue residue;
    Advice advice;
    Hook next;

    public Hook(Residue r, Advice a) {
        residue = r;
        advice = a;
        next = null;
    }

    public Residue getResidue() {
        return residue;
    }

    public Advice getAdvice() {
        return advice;
    }

    public static class List {
        Hook list = null;
        Hook tail = null;

        public void add(Residue r, Advice a) {
            Hook h = new Hook(r, a);
            if (tail == null)
                list = h;
            else
                tail.next = h;

            tail = h;
        }

        public boolean notEmpty() {
            return list != null;
        }

        public Iterator iterator() {
            return new Iterator(list);
        }
    }

    public static class Iterator implements java.util.Iterator {
        Hook hook;

        Iterator(Hook h) {
            hook = h;
        }

        public Iterator copy() {
            return new Iterator(hook);
        }

        public boolean hasNext() {
            return hook != null;
        }

        /**
         * Returns the next Hook object.
         */
        public Object next() {
            Hook h = hook;
            hook = hook.next;
            return h;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
