/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

/**
 * A pointcut residue.  A Java expression evaluated at runtime
 * for determining whether or not an advice is executed.
 */
public class Residue {
    private String condition;

    public Residue(String ifExpr) {
        condition = ifExpr;
    }

    public String getIf() { return condition; }

    public static Residue or(Residue left, Residue right) {
        // null means true.
        if (left == null || right == null)
            return null;
        else
            return concat(left, right, ") || (");
    }

    public static Residue and(Residue left, Residue right) {
        if (left == null)
            return right;
        else if (right == null)
            return left;
        else
            return concat(left, right, ") && (");
    }

    public static Residue not(Residue expr) {
        expr.condition = "!(" + expr.condition + ")";
        return expr;
    }

    private static Residue concat(Residue left, Residue right, String op) {
        left.condition = "(" + left.condition + op + right.condition + ")";
        return left;
    }
}
